
@import "mixins";

// Global namespace: harambe-*


.responsive-image {
    @include responsive-image;
    border-radius: 4px;
}

.navbar-fixed-top {

}

.image-circle {
  border-radius: 50%;
}

// img.profile-avatar
img {
  &.harambe-profile-avatar {
      @include profile-avatar(100px);
    &-30{
      @include profile-avatar(30px);
    }
    &-40{
      @include profile-avatar(40px);
    }
    &-50{
      @include profile-avatar(50px);
    }
    &-100{
      @include profile-avatar(100px);
    }
    &-150{
      @include profile-avatar(150px);
    }
    &-200{
      @include profile-avatar(200px);
    }
    &-250{
      @include profile-avatar(250px);
    }
  }
}

.harambe {
  &-share-buttons {
    font-size: 20px;
    a {
      text-decoration: none !important;
    }
  }
}

// navbar change hamburger into X when collapsed
.navbar-toggle {
  border: none;
  background: transparent !important;

  &:hover {
    background: transparent !important;
  }

  .icon-bar {
    width: 22px;
    transition: all 0.2s;
  }
  .top-bar {
    transform: rotate(45deg);
    transform-origin: 10% 10%;
  }

  .middle-bar {
    opacity: 0;
  }
  .bottom-bar {
    transform: rotate(-45deg);
    transform-origin: 10% 90%;
  }

  &.collapsed {
    .top-bar {
      transform: rotate(0);
    }
    .middle-bar {
      opacity: 1;
    }
    .bottom-bar {
      transform: rotate(0);
    }
  }
}


//
textarea.form-control {
  height: auto;
  min-height: 100px;
}
